package com.diagbot.facade;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.client.CRFServiceClient;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.model.ai.AIAnalyze;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.model.label.VitalLabel;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.SearchData;
import com.diagbot.vo.StandConvert;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 通过业务facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class CommonFacade {
    @Autowired
    CRFServiceClient crfServiceClient;

    //组装好的label
    public WordCrfDTO crf_process(SearchData searchData){
        
        AIAnalyze aiAnalyze = new AIAnalyze(crfServiceClient);
        WordCrfDTO wordCrfDTO = new WordCrfDTO();
        wordCrfDTO.setAge(searchData.getAge());
        wordCrfDTO.setSex(searchData.getSex());
        wordCrfDTO.setLis(searchData.getLis());
        wordCrfDTO.setPacs(searchData.getPacs());
        wordCrfDTO.setLisOrder(searchData.getLisOrder());
        wordCrfDTO.setPacsOrder(searchData.getPacsOrder());
        wordCrfDTO.setDrugOrder(searchData.getDrugOrder());
        aiAnalyze.aiProcess(searchData, wordCrfDTO);
        return wordCrfDTO;
    }

    //返回给图谱的词，这些词需要转换成标准词
    public StandConvert dataTypeGet(WordCrfDTO wordCrfDTO){
        StandConvert standConvert = new StandConvert();

        //所有的症状(主诉、现病史)
        List<String> clinicalList = new ArrayList<>();
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        clinicalList.addAll(CoreUtil.getPropertyList(chiefLabel.getClinicals()));
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        clinicalList.addAll(CoreUtil.getPropertyList(presentLabel.getClinicals()));

        //所有的疾病(主诉、现病史,诊断列表,过去史)
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();
        List<String> diagList = new ArrayList<>();
        diagList.addAll(CoreUtil.getPropertyList(chiefLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(presentLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(diagLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(pastLabel.getDiags()));

        //所有化验(结构化数据)
        List<Lis> lis = wordCrfDTO.getLis();
        if(ListUtil.isNotEmpty(lis)){
            List<String> lis_unique = lis.stream().filter(x -> StringUtils.isBlank(x.getUniqueName()))
                    .map(x ->x.getName()+x.getDetailName()).collect(Collectors.toList());
            standConvert.setLisList(lis_unique);
        }

        //所有辅助检查(结构化数据)
        List<Pacs> pacs = wordCrfDTO.getPacs();
        if(ListUtil.isNotEmpty(pacs)){
            List<String> pacss_unique = pacs.stream().filter(x -> StringUtils.isBlank(x.getUniqueName()))
                    .map(x ->x.getName()).collect(Collectors.toList());
            standConvert.setPacsList(pacss_unique);
        }

        //所有药品(现病史、既往史)
        List<String> drugList = new ArrayList<>();
        //1、现病史中的药品
        drugList.addAll(CoreUtil.getPropertyList(presentLabel.getMedicines()));
        //2、既往史(药物过敏)
        drugList.addAll(CoreUtil.getPropertyList(pastLabel.getAllergyMedicines()));
        standConvert.setDrugList(drugList);

        //所有手术(现病史、既往史)
        List<String> operationList = new ArrayList<>();
        //1、现病史中的手术
        operationList.addAll(CoreUtil.getPropertyList(presentLabel.getOperations()));
        //2、既往史中的手术
        operationList.addAll(CoreUtil.getPropertyList(pastLabel.getOperations()));
        standConvert.setOperationList(operationList);

        //所有体征
        List<String> vitalList = new ArrayList<>();
        VitalLabel vitalLabel = wordCrfDTO.getVitalLabel();
        vitalList.addAll(CoreUtil.getPropertyList(vitalLabel.getVitals()));
        standConvert.setVitalList(vitalList);


        standConvert.setClinicalList(clinicalList);
        standConvert.setDiaglList(diagList);
        return standConvert;
    }

    //把图谱返回的标准词set到label中
    public void dataTypeSet(WordCrfDTO wordCrfDTO, Map<Integer, Map<String, String>> map){
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        List<Lis> lis = wordCrfDTO.getLis();
        List<Pacs> pacs = wordCrfDTO.getPacs();
        VitalLabel vitalLabel = wordCrfDTO.getVitalLabel();

        //症状回填
        CoreUtil.setPropertyList(chiefLabel.getClinicals(),map.get(StandConvertEnum.clinical.getKey()));
        CoreUtil.setPropertyList(presentLabel.getClinicals(),map.get(StandConvertEnum.clinical.getKey()));
        //诊断回填
        CoreUtil.setPropertyList(diagLabel.getDiags(),map.get(StandConvertEnum.disease.getKey()));
        //药品回填
        CoreUtil.setPropertyList(presentLabel.getMedicines(),map.get(StandConvertEnum.drug.getKey()));
        CoreUtil.setPropertyList(pastLabel.getAllergyMedicines(),map.get(StandConvertEnum.drug.getKey()));
        // TODO: 2020/8/5 化验回填
        CoreUtil.setPropertyList(lis,"name","detailName","uniqueName",map.get(StandConvertEnum.lis.getKey()));
        // TODO: 2020/8/5 辅助检查回填
        CoreUtil.setPropertyList(pacs,"name","uniqueName",map.get(StandConvertEnum.pacs.getKey()));
        //体征回填
        CoreUtil.setPropertyList(vitalLabel.getVitals(),map.get(StandConvertEnum.vital.getKey()));
        //手术回填
        CoreUtil.setPropertyList(presentLabel.getOperations(),map.get(StandConvertEnum.operation.getKey()));
        CoreUtil.setPropertyList(pastLabel.getOperations(),map.get(StandConvertEnum.operation.getKey()));
    }

}
