package com.diagbot.facade;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.LisBillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.entity.node.Disease;
import com.diagbot.entity.node.LIS;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.PACS;
import com.diagbot.repository.DiseaseRepository;
import com.diagbot.repository.LISNode;
import com.diagbot.repository.LisRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.PACSNode;
import com.diagbot.repository.PacsRepository;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.PushNeoVO;
import com.diagbot.vo.StandConvert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsRepository pacsRepository;
    @Autowired
    LisRepository lisRepository;
    @Autowired
    DiseaseRepository diseaseRepository;


    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();
        res.put("泰舒达类", Arrays.asList("泰舒达"));
        res.put("青霉素类", Arrays.asList("青霉素"));
        return res;
    }

    /**
     * 通过诊断反推
     *
     * @return
     */
    public Map<String, Object> getInverse(PushNeoVO pushNeoVO) {
        Map<String, Object> map = new LinkedHashMap<>();

        return map;
    }

    /**
     * 处理开单合理性图谱数据
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
        billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
        billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));
        getDiag(billNeoVO.getDiagList());

        for (BillNeoDTO billNeoDTO:billNeoDTOs) {
            System.out.println(billNeoDTO);
        }
        return billNeoDTOs;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        MedicineNode medicineNode = new MedicineNode();
        Medicine medicine;
        String medname;

        for (String term : medsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            medname = medsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine != null) {
                drugBillNeoDTO = medicineNode.MedicinetoDrugDTO(medicine);
            }

            billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }


        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        PACSNode pacsNode = new PACSNode();
        PACS pacs;
        String pacsname;

        for (String term : pacsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            pacsname = pacsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacs = pacsRepository.findByNameIs(pacsname);

            if (pacs != null) {
                pacsBillNeoDTO = pacsNode.PacstoPACSDTO(pacs);
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisBill(Map<String, String> listerm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        LisBillNeoDTO lisBillNeoDTO;

        LISNode lisNode = new LISNode();
        LIS lis;
        String lisname;

        for (String term : listerm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            lisname = listerm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(lisname);

            lisBillNeoDTO = new LisBillNeoDTO();

            lis = lisRepository.findByNameIs(lisname);

            if (lis != null) {
                lisBillNeoDTO = lisNode.ListoLISDTO(lis);
            }

            billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    public void getDiag(Map<String, String> diaglist) {

        Disease disease;
        String diag;

        for (String term : diaglist.keySet()) {

            diag = diaglist.get(term);
            disease = diseaseRepository.findByNameIs(diag);

            if (disease != null) {
                System.out.println(disease);
            }
        }


    }


    /**
     * 标准词转换
     *
     * @param standConvert
     *
     * @return  Map<String, Map<String, String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<Integer, Map<String, String>> standConvert(StandConvert standConvert) {
        // TODO 待处理业务
        return new HashMap<>();
    }

}
